@@ -1,3 +1,5 @@
+//! Various C++-related types and functions needed for the `cpp_to_rust` project.
+
 #[cfg(test)]
 mod tests {
   use std::rc::Rc;
@@ -21,9 +23,7 @@ mod tests {
   #[test]
   fn test_drop_calls_deleter() {
     let value1 = Rc::new(RefCell::new(10));
-    let mut object1 = Struct1 {
-      value: value1.clone()
-    };
+    let mut object1 = Struct1 { value: value1.clone() };
     assert!(value1.borrow().clone() == 10);
     unsafe {
       CppBox::new(&mut object1 as *mut _);
@@ -49,10 +49,9 @@ pub type Deleter<T> = unsafe extern "C" fn(this_ptr: *mut T);
 ///   }
 /// }
 /// ```
-pub trait CppDeletable : Sized {
+pub trait CppDeletable: Sized {
   /// Returns deleter function for this type.
   fn deleter() -> Deleter<Self>;
-
 }
 
 /// A C++ pointer wrapper to manage deletion of objects.
@@ -60,15 +59,17 @@ pub trait CppDeletable : Sized {
 /// Objects of CppBox should be created by calling into_box() for
 /// types that implement CppDeletable trait. The object will
 /// be deleted when corresponding CppBox is deleted.
-pub struct CppBox<T> {
+pub struct CppBox<T: CppDeletable> {
   ptr: *mut T,
   deleter: Deleter<T>,
 }
 
-impl<T> CppBox<T> {
+impl<T: CppDeletable> CppBox<T> {
+  /// Returns constant raw pointer to the value in the box.
   pub fn as_ptr(&self) -> *const T {
     self.ptr
   }
+  /// Returns mutable raw pointer to the value in the box.
   pub fn as_mut_ptr(&self) -> *mut T {
     self.ptr
   }
@@ -80,6 +81,11 @@ impl<T> CppBox<T> {
     self.ptr = std::ptr::null_mut();
     ptr
   }
+
+  /// Returns true if the pointer is null.
+  pub fn is_null(&self) -> bool {
+    self.ptr.is_null()
+  }
 }
 
 impl<T: CppDeletable> CppBox<T> {
@@ -87,8 +93,8 @@ impl<T: CppDeletable> CppBox<T> {
   ///
   /// You should use this function only for
   /// pointers that were created on C++ side and passed through
-  /// a FFI boundary to Rust. An object created with C++ "new"
-  /// must be deleted using C++ "delete", which is executed by CppBox.
+  /// a FFI boundary to Rust. An object created with C++ `new`
+  /// must be deleted using C++ `delete`, which is executed by `CppBox`.
   ///
   /// Do not use this function for objects created in memory managed by Rust.
   /// Any wrapper constructor or function that returns an owned object
@@ -96,51 +102,51 @@ impl<T: CppDeletable> CppBox<T> {
   ///
   /// Do not use this function for objects that would be deleted by other means.
   /// If another C++ object is the owner of the passed object,
-  /// it will attempt to delete it. Together with CppBox, it would result
-  /// in a double deletion, which should never happen.
+  /// it will attempt to delete it. If `CppBox` containing the object still exists,
+  /// it would result in a double deletion, which should never happen.
+  ///
+  /// Use `CppBox::into_raw` to unwrap the pointer before passing it to
+  /// a function that takes ownership of the object.
+  ///
+  /// It's permitted to put a null pointer into a `CppBox`. Deleter function
+  /// will not be called for a null pointer. However, attempting to dereference
+  /// a null pointer in a `CppBox`
+  /// using `as_ref`, `as_mut`, `deref` or `deref_mut` will result in a panic.
   pub unsafe fn new(ptr: *mut T) -> CppBox<T> {
     CppBox {
       ptr: ptr,
-      deleter: CppDeletable::deleter()
+      deleter: CppDeletable::deleter(),
     }
   }
 }
 
 
-impl<T> AsRef<T> for CppBox<T> {
+impl<T: CppDeletable> AsRef<T> for CppBox<T> {
   fn as_ref(&self) -> &T {
-    unsafe {
-      &*self.ptr
-    }
+    unsafe { self.ptr.as_ref().unwrap() }
   }
 }
 
-impl<T> AsMut<T> for CppBox<T> {
+impl<T: CppDeletable> AsMut<T> for CppBox<T> {
   fn as_mut(&mut self) -> &mut T {
-    unsafe {
-      &mut *self.ptr
-    }
+    unsafe { self.ptr.as_mut().unwrap() }
   }
 }
 
-impl<T> std::ops::Deref for CppBox<T> {
+impl<T: CppDeletable> std::ops::Deref for CppBox<T> {
   type Target = T;
   fn deref(&self) -> &T {
-    unsafe {
-      &*self.ptr
-    }
+    unsafe { self.ptr.as_ref().unwrap() }
   }
 }
 
-impl<T> std::ops::DerefMut for CppBox<T> {
+impl<T: CppDeletable> std::ops::DerefMut for CppBox<T> {
   fn deref_mut(&mut self) -> &mut T {
-    unsafe {
-      &mut *self.ptr
-    }
+    unsafe { self.ptr.as_mut().unwrap() }
   }
 }
 
-impl<T> Drop for CppBox<T> {
+impl<T: CppDeletable> Drop for CppBox<T> {
   fn drop(&mut self) {
     if !self.ptr.is_null() {
       unsafe {
@@ -150,25 +156,25 @@ impl<T> Drop for CppBox<T> {
   }
 }
 
-/// Additional argument for a function that returns a C++ object as Rust struct.
-pub struct AsStruct;
+impl<T: CppDeletable> Default for CppBox<T> {
+  fn default() -> CppBox<T> {
+    CppBox {
+      ptr: std::ptr::null_mut(),
+      deleter: CppDeletable::deleter(),
+    }
+  }
+}
 
-/// Additional argument for a function that returns a C++ object as a pointer
-/// enclosed in a CppBox.
-pub struct AsBox;
-
-/// This module contains `NewUninitialized` trait which should not be used directly.
+/// This module contains `NewUninitialized` trait.
+/// It's an implementation detail of `cpp_to_rust` and should not be used directly.
 pub mod new_uninitialized {
 
   /// A trait for types that can be created with
   /// uninitialized internal buffer.
   ///
-  /// This trait should not be used directly.
-  /// It's only useful for the code automatically generated
-  /// by cpp_to_rust.
+  /// This trait is an implementation detail of `cpp_to_rust` and should not be used directly.
   pub trait NewUninitialized {
-    /// Creates new object with
-    /// uninitialized internal buffer.
+    /// Creates new object with uninitialized internal buffer.
     unsafe fn new_uninitialized() -> Self;
   }
 }
@@ -202,13 +208,17 @@ pub trait StaticCast<T> {
 /// Converts type of a const pointer using `StaticCast` implementation of the type.
 /// If `ptr` is null, this function does nothing and returns null pointer.
 pub fn static_cast<R, T: StaticCast<R>>(ptr: *const T) -> *const R {
-  unsafe { ptr.as_ref() }.map(|x| x.static_cast() as *const R).unwrap_or(std::ptr::null())
+  unsafe { ptr.as_ref() }
+    .map(|x| x.static_cast() as *const R)
+    .unwrap_or(std::ptr::null())
 }
 
 /// Converts type of a mutable pointer using `StaticCast` implementation of the type.
 /// If `ptr` is null, this function does nothing and returns null pointer.
 pub fn static_cast_mut<R, T: StaticCast<R>>(ptr: *mut T) -> *mut R {
-  unsafe { ptr.as_mut() }.map(|x| x.static_cast_mut() as *mut R).unwrap_or(std::ptr::null_mut())
+  unsafe { ptr.as_mut() }
+    .map(|x| x.static_cast_mut() as *mut R)
+    .unwrap_or(std::ptr::null_mut())
 }
 
 /// Provides access to C++ `static_cast` conversion from base class to derived class.
@@ -244,7 +254,10 @@ pub trait UnsafeStaticCast<T> {
 /// or a class derived from `R`.
 /// If `ptr` is null, this function does nothing and returns null pointer.
 pub unsafe fn unsafe_static_cast<R, T: UnsafeStaticCast<R>>(ptr: *const T) -> *const R {
-  ptr.as_ref().map(|x| x.static_cast() as *const R).unwrap_or(std::ptr::null())
+  ptr
+    .as_ref()
+    .map(|x| x.static_cast() as *const R)
+    .unwrap_or(std::ptr::null())
 }
 
 /// Converts type of a mutable pointer using `UnsafeStaticCast` implementation of the type.
@@ -252,7 +265,10 @@ pub unsafe fn unsafe_static_cast<R, T: UnsafeStaticCast<R>>(ptr: *const T) -> *c
 /// or a class derived from `R`.
 /// If `ptr` is null, this function does nothing and returns null pointer.
 pub unsafe fn unsafe_static_cast_mut<R, T: UnsafeStaticCast<R>>(ptr: *mut T) -> *mut R {
-  ptr.as_mut().map(|x| x.static_cast_mut() as *mut R).unwrap_or(std::ptr::null_mut())
+  ptr
+    .as_mut()
+    .map(|x| x.static_cast_mut() as *mut R)
+    .unwrap_or(std::ptr::null_mut())
 }
 
 
@@ -286,7 +302,11 @@ pub trait DynamicCast<T> {
 /// a class derived from `R`.
 /// If `ptr` is null, this function does nothing and returns null pointer.
 pub unsafe fn dynamic_cast<R, T: DynamicCast<R>>(ptr: *const T) -> *const R {
-  ptr.as_ref().and_then(|x| x.dynamic_cast()).map(|x| x as *const R).unwrap_or(std::ptr::null())
+  ptr
+    .as_ref()
+    .and_then(|x| x.dynamic_cast())
+    .map(|x| x as *const R)
+    .unwrap_or(std::ptr::null())
 }
 
 /// Converts type of a mutable pointer using `DynamicCast` implementation of the type.
@@ -295,15 +315,9 @@ pub unsafe fn dynamic_cast<R, T: DynamicCast<R>>(ptr: *const T) -> *const R {
 /// Returns null pointer if `ptr` does not point to an instance of `R`.
 /// If `ptr` is null, this function does nothing and returns null pointer.
 pub unsafe fn dynamic_cast_mut<R, T: DynamicCast<R>>(ptr: *mut T) -> *mut R {
-  ptr.as_mut().and_then(|x| x.dynamic_cast_mut()).map(|x| x as *mut R).unwrap_or(std::ptr::null_mut())
+  ptr
+    .as_mut()
+    .and_then(|x| x.dynamic_cast_mut())
+    .map(|x| x as *mut R)
+    .unwrap_or(std::ptr::null_mut())
 }
-
-
-
-
-
-
-
-
-
-
